\name{Maxwell}
\alias{Maxwell}
\alias{dmaxwell}
\alias{pmaxwell}
\alias{qmaxwell}
\alias{rmaxwell}
\title{The Maxwell Distribution}
\description{
  Density, and distribution function
  for the Maxwell distribution.

}
\usage{
dmaxwell(x, a)
pmaxwell(q, a)
qmaxwell(p, a)
rmaxwell(n, a)
}
\arguments{
  \item{x, q}{vector of quantiles.}
  \item{p}{vector of probabilities.}
  \item{n}{number of observations. A single positive integer. }
  \item{a}{the parameter.}
}
\value{
  \code{dmaxwell} gives the density,
  \code{pmaxwell} gives the distribution function,
  \code{qmaxwell} gives the quantile function, and
  \code{rmaxwell} generates random deviates.
}
\references{
  Balakrishnan, N. and Nevzorov, V. B. (2003) 
  \emph{A Primer on Statistical Distributions}.
  Hoboken, New Jersey: Wiley.

}
\author{ T. W. Yee }
\details{
  See \code{\link{maxwell}}, the \pkg{VGAM} family function
  for estimating the parameter \eqn{a} by maximum likelihood estimation,
  for the formula of the probability density function.
}
\note{
  The Maxwell distribution is related to the Rayleigh distribution.
}
\seealso{
  \code{\link{maxwell}},
  \code{\link{Rayleigh}},
  \code{\link{rayleigh}}.
}
\examples{
\dontrun{
a = 3
x = seq(-0.5, 3, len=100)
plot(x, dmaxwell(x, a=a), type="l", col="blue", las=1, ylab="",
     main="blue is density, red is cumulative distribution function",
     sub="Purple lines are the 10,20,...,90 percentiles")
abline(h=0, col="blue", lty=2)
lines(x, pmaxwell(x, a=a), type="l", col="red")
probs = seq(0.1, 0.9, by=0.1)
Q = qmaxwell(probs, a=a)
lines(Q, dmaxwell(Q, a), col="purple", lty=3, type="h")
lines(Q, pmaxwell(Q, a), col="purple", lty=3, type="h")
abline(h=probs, col="purple", lty=3)
pmaxwell(Q, a) - probs    # Should be all zero
}
}
\keyword{distribution}


